import os, pathlib
import cv2
import gradio as gr
import torch
from basicsr.archs.srvgg_arch import SRVGGNetCompact
from gfpgan.utils import GFPGANer
from realesrgan.utils import RealESRGANer
os.system("hub install deoldify==1.0.1")
import paddlehub as hub
os.system("pip freeze")


# download weights
if not os.path.exists('GFPGANv1.4.pth'):
    os.system("wget https://github.com/TencentARC/GFPGAN/releases/download/v1.3.0/GFPGANv1.4.pth -P .")

model = SRVGGNetCompact(num_in_ch=3, num_out_ch=3, num_feat=64, num_conv=32, upscale=4, act_type='prelu')
model_path = 'realesr-general-x4v3.pth'
half = True if torch.cuda.is_available() else False
upsampler = RealESRGANer(scale=4, model_path=model_path, model=model, tile=0, tile_pad=10, pre_pad=0, half=half)

os.makedirs('output', exist_ok=True)

colorizer = hub.Module(name='deoldify')
render_factor=5

def colorize_image(image):
    color_image = colorizer.predict(image)
    return color_image


def inference(img, scale):
    print(img, scale)
    try:
        extension = os.path.splitext(os.path.basename(str(img)))[1]
        img = cv2.imread(img, cv2.IMREAD_UNCHANGED)
        if len(img.shape) == 3 and img.shape[2] == 4:
            img_mode = 'RGBA'
        elif len(img.shape) == 2:  # for gray inputs
            img_mode = None
            img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
        else:
            img_mode = None

        h, w = img.shape[0:2]
        if h < 300:
            img = cv2.resize(img, (w * 2, h * 2), interpolation=cv2.INTER_LANCZOS4)
        face_enhancer = GFPGANer(model_path='GFPGANv1.4.pth', upscale=2, arch='clean', channel_multiplier=2, bg_upsampler=upsampler)
        try:
            # _, _, output = face_enhancer.enhance(img, has_aligned=False, only_center_face=False, paste_back=True, weight=weight)
            _, _, output = face_enhancer.enhance(img, has_aligned=False, only_center_face=False, paste_back=True)
        except RuntimeError as error:
            print('Error', error)

        try:
            if scale != 2:
                interpolation = cv2.INTER_AREA if scale < 2 else cv2.INTER_LANCZOS4
                h, w = img.shape[0:2]
                output = cv2.resize(output, (int(w * scale / 2), int(h * scale / 2)), interpolation=interpolation)
        except Exception as error:
            print('wrong scale input.', error)
        if img_mode == 'RGBA':  # RGBA images should be saved in png format
            extension = 'png'
        else:
            extension = 'jpg'
        save_path = f'output/out.{extension}'
        output = cv2.cvtColor(output, cv2.COLOR_BGR2RGB)
        cv2.imwrite(save_path, output)
        print('upsampled image!')
        output_img = colorize_image(save_path)
        outputim = output_img[1]
        print(type(outputim))
        print('colorized image!')
        return pathlib.Path(outputim)
    except Exception as error:
        print('global exception', error)
        return None, None



with gr.Blocks(css = "footer {visibility: hidden}" ) as demo:
    with gr.Row():
        with gr.Column():
            in_image = gr.inputs.Image(type="filepath", label="Input")
            factor = gr.Slider(2, 10, value=2, step = 1, label="Rescaling factor")
            #gr.inputs.Number(, default=2)
            btn = gr.Button("着色")
        with gr.Column():
            gallery = gr.Image()
            #out_image = gr.outputs.Image(type="numpy", label="Output (The whole image)")
            #dld = gr.outputs.File(label="Download the output image")
    btn.click(fn=inference, inputs=[in_image,factor], outputs=gallery)

demo.launch(server_name="0.0.0.0", share=True)
